/*
    Copyright (c) 2002-2009 Tampere University of Technology.

    This file is part of TTA-Based Codesign Environment (TCE).

    Permission is hereby granted, free of charge, to any person obtaining a
    copy of this software and associated documentation files (the "Software"),
    to deal in the Software without restriction, including without limitation
    the rights to use, copy, modify, merge, publish, distribute, sublicense,
    and/or sell copies of the Software, and to permit persons to whom the
    Software is furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
    THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
    DEALINGS IN THE SOFTWARE.
 */
/**
 * @file ConstantNode.cc
 *
 * Implementation of ConstantNode class.
 *
 * @author Mikael Lepist 2008 (mikael.lepisto-no.spam-tut.fi)
 * @note rating: red
 */

#include "ConstantNode.hh"
#include "Conversion.hh"

/**
 * Constructor.
 */
ConstantNode::ConstantNode(int value) :
    OperationDAGNode(), value_(value) {}

/**
 * Cpoy constructor.
 */
ConstantNode::ConstantNode(const ConstantNode& other) :
    OperationDAGNode(), value_(other.value()) {}

/**
 * Clones node. Needed for dynamic binding, when copying 
 * instances through the base class.
 *
 * @return Dynamically allocated copy of the node. 
 */
GraphNode* 
ConstantNode::clone() const {
    return new ConstantNode(*this);
}

int 
ConstantNode::value() const {
    return value_;
}

std::string 
ConstantNode::toString() const {
    return "(" + Conversion::toString(value_) + ")";
}
